/******************************************************************************
* C Source: Plate                                                             *
*                                                                             *
* File:                                                                       *
*    plate.c                                                                  *
*                                                                             *
* Description:                                                                *
*    This set of routines manipulate a structure (see plate.h) which contains *
*    data about a single plate, namely, it's id, region, center right         *
*    ascension and declination in radians, epoch, and number of it's CD_ROM.  *
*                                                                             *
* Module List:                                                                *
*    Compare_Plates                                                           *
*       Lexigraphically compares the ids of two plates.                       *
*    Create_Plate                                                             *
*       Create memory for a plate and inserts its data.                       *
*    Delete_Plate                                                             *
*       Free memory for a plate.                                              *
*    Plate_Directory_Exists:                                                  *
*       Checks if the directory for a plate's image data exists.              *
*    Plates_Disc_Is_Available:                                                *
*       Returns TRUE if the disc for the plate is available, ie, the number   *
*       of the disc is not zero. If it is zero, then FALSE is returned.       *
*    Read_Plate_Info:                                                         *
*       Read data for a single Plate from a file.                             *
*                                                                             *
* History:                                                                    *
*    Created    : 15-JUN-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
*                                                                             *
*    Updated    : 14-NOV-94 by J. Doggett,                                    *
*       Updated Read_Plate_Info function.                                     *
*    Redelivered: 30-NOV-94 by J. Doggett,                                    *
*       GetImage Version 1.1.                                                 *
*                                                                             *
* Copyright (c) 1993, 1994, Association of Universities for Research in       *
* Astronomy, Inc.  All Rights Reserved.                                       *
* Produced under National Aeronautics and Space Administration grant          *
* NAG W-2166.                                                                 *
******************************************************************************/
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

/* access function not prototyped in standard place */
#if defined __VMS
#  include <unixio.h>
#elif defined __UNIX
#  include <unixstd.h>
#else
#endif

#include "plate.h"

#include "astronmy.h"
#include "filenams.h"
#include "generic.h"

/******************************************************************************
* Module:                                                                     *
*    Compare_Plates                                                           *
*                                                                             *
* File:                                                                       *
*    plate.c                                                                  *
*                                                                             *
* Description:                                                                *
*    Compares the plate id's of two plates using strcmp.                      *
*                                                                             *
* Return Value:                                                               *
*    int: The result from using strcmp to compare the two plate ids.          *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       Plate_1                                                               *
*          PLATE, Reference                                                   *
*             The first plate to compare.                                     *
*       Plate_2                                                               *
*          PLATE, Reference                                                   *
*             The second plate to compare.                                    *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 15-JUN-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/******************************
* Compare_Plates declarations *
******************************/
int   Compare_Plates( Plate_1, Plate_2 )
PLATE Plate_1;
PLATE Plate_2;
{
/***************************************
* Compare_Plates begin executable code *
***************************************/
   return strcmp( Plate_1->Id, Plate_2->Id );
}
/*************************************
* Compare_Plates end executable code *
*************************************/

/******************************************************************************
* Module:                                                                     *
*    Create_Plate                                                             *
*                                                                             *
* File:                                                                       *
*    plate.c                                                                  *
*                                                                             *
* Description:                                                                *
*    Allocates memory to hold data for a plate, and inserts the data into the *
*    memory.                                                                  *
*                                                                             *
* Return Value:                                                               *
*    PLATE: The newly created plate.                                          *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       Id                                                                    *
*          STRING, Reference                                                  *
*             The plate id for the plate.                                     *
*       Region                                                                *
*          STRING, Reference                                                  *
*             The region id for the plate.                                    *
*       Center_RA                                                             *
*          double, Value                                                      *
*             The right ascension of the plate center in radians.             *
*       Center_Dec                                                            *
*          double, Value                                                      *
*             The declination of the plate center in radians.                 *
*       Epoch                                                                 *
*          double, Value                                                      *
*             The epoch at which the plate was taken.                         *
*       Source                                                                *
*          STRING, Reference                                                  *
*             ID for telescope where plate was taken (UK or PAL)              *
*       Disc                                                                  *
*          int, Value                                                         *
*             The number of the disc in the CD-ROM set where the plate        *
*             image data is located.                                          *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 15-JUN-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/****************************
* Create_Plate declarations *
****************************/
PLATE  Create_Plate( Id, Region, Center_RA, Center_Dec, Epoch, Source, Disc )
STRING Id        ;
STRING Region    ;
double Center_RA ;
double Center_Dec;
double Epoch     ;
STRING Source    ;
int    Disc      ;

/**********************
* Create_Plate page 2 *
**********************/

{
   PLATE New_Plate; /* Newly created plate structure         */
   int   Nth_Char ; /* For loop index for copying characters */

/*************************************
* Create_Plate begin executable code *
*************************************/
   New_Plate = ( PLATE )Allocate_Memory( sizeof( *New_Plate ) );

   strcpy( New_Plate->Id, Id );
   strcpy( New_Plate->Region, Region );
   New_Plate->Center_RA  = Center_RA  ;
   New_Plate->Center_Dec = Center_Dec ;
   New_Plate->Epoch      = Epoch      ;
   strcpy( New_Plate->Source, Source) ;
   New_Plate->Disc       = Disc       ;

   return New_Plate;
}
/***********************************
* Create_Plate end executable code *
***********************************/

/******************************************************************************
* Module:                                                                     *
*    Delete_Plate                                                             *
*                                                                             *
* File:                                                                       *
*    plate.c                                                                  *
*                                                                             *
* Description:                                                                *
*    Free memory previously allocated for a plate.                            *
*                                                                             *
* Return Value:                                                               *
*    none                                                                     *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       Plate                                                                 *
*          PLATE, Reference                                                   *
*             The plate we're deleting from memory.                           *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 15-JUN-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/****************************
* Delete_Plate declarations *
****************************/
void  Delete_Plate( Plate )
PLATE Plate;
{
/*************************************
* Delete_Plate begin executable code *
*************************************/
   if ( Plate != NULL )
      free( Plate );

   return;
}
/***********************************
* Delete_Plate end executable code *
***********************************/

/******************************************************************************
* Module:                                                                     *
*    Plate_Directory_Exists                                                   *
*                                                                             *
* File:                                                                       *
*    plate.c                                                                  *
*                                                                             *
* Description:                                                                *
*    Checks if the directory for a plate's image data exists.                 *
*                                                                             *
* Return Value:                                                               *
*    BOOL: TRUE if the directory exists, FALSE otherwise.                     *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       Plate_Name                                                            *
*          STRING, Reference                                                  *
*             Name of plate for whose's directory we're looking.              *
*       Image_Root_Directory                                                  *
*          STRING, Reference                                                  *
*             Name of parent directory for all plates' directories.           *
*    Input/Output Arguments:                                                  *
*          none                                                               *
*    Output Arguments:                                                        *
*          none                                                               *
*                                                                             *
* History:                                                                    *
*    Created    : 07-SEP-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/**************************************
* Plate_Directory_Exists declarations *
**************************************/
BOOL   Plate_Directory_Exists( Plate_Name, Image_Root_Directory )
STRING Plate_Name          ;
STRING Image_Root_Directory;
{
   STRING Temp_File_Name     ; /* Temp place for base directory file name */
   STRING Directory_File_Name; /* Full name for directory file            */
   FILE   *File              ; /* Pointer to file when we try to open it  */
   BOOL   Exists             ; /* Holds the return value                  */

/***********************************************
* Plate_Directory_Exists begin executable code *
***********************************************/
#if defined __VMS
   Temp_File_Name = ( STRING )Allocate_Memory( strlen( Plate_Name ) + 5 );
   strcpy( Temp_File_Name, Plate_Name );
   strcat( Temp_File_Name, ".DIR" );

#elif defined __UNIX
   Temp_File_Name = ( STRING )Allocate_Memory( strlen( Plate_Name ) + 1 );
   strcpy( Temp_File_Name, Plate_Name );

#else
   fprintf(stderr, "Unknown file system error in Plate_Diretory_Exits");
   exit(EXIT_FAILURE);


/********************************
* Plate_Directory_Exists page 2 *
********************************/

#endif

   Directory_File_Name = NULL;
   if (Set_FileNames_Case(Image_Root_Directory, Temp_File_Name)) {
      Directory_File_Name =
              Build_Full_File_Name( Image_Root_Directory, "", Temp_File_Name );

      if ( access( Directory_File_Name, 0 ) == 0 )
         Exists = TRUE;
      else
         Exists = FALSE;
   }
   else
      Exists = FALSE;

   free( Temp_File_Name );
   if (Directory_File_Name != NULL)
      free( Directory_File_Name );

   return Exists;
}
/*********************************************
* Plate_Directory_Exists end executable code *
*********************************************/

/******************************************************************************
* Module:                                                                     *
*    Plates_Disc_Is_Available                                                 *
*                                                                             *
* File:                                                                       *
*    plate.c                                                                  *
*                                                                             *
* Description:                                                                *
*       Returns TRUE if the disc for the plate is available, ie, the number   *
*       of the disc is not zero. If it is zero, then FALSE is returned.       *
*                                                                             *
* Return Value:                                                               *
*    BOOL: TRUE if the disc is available for the input plate, Otherwise,      *
*       FALSE.                                                                *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       Plate                                                                 *
*          PLATE, Reference                                                   *
*             The plate whose disc we're checking for.                        *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 07-SEP-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/****************************************
* Plates_Disc_Is_Available declarations *
****************************************/
BOOL  Plates_Disc_Is_Available( Plate )
PLATE Plate;
{
/*************************************************
* Plates_Disc_Is_Available begin executable code *
*************************************************/
   if ( Plate->Disc != 0 )
      return TRUE;
   else
      return FALSE;
}
/***********************************************
* Plates_Disc_Is_Available end executable code *
***********************************************/

/******************************************************************************
* Module:                                                                     *
*    Read_Plate_Info                                                          *
*                                                                             *
* File:                                                                       *
*    plate.c                                                                  *
*                                                                             *
* Description:                                                                *
*    Read data for Plate from a file.                                         *
*                                                                             *
* Return Value:                                                               *
*    PLATE: The plate read from the file.                                     *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       InputFile                                                             *
*          FILE*, Value                                                       *
*             File pointer to the input file.                                 *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 17-DEC-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
*                                                                             *
*    Updated    : 01-JUN-94 by J. Doggett,                                    *
*       Modified for the change in format of the input file from records      *
*       separated with a new-line character to records separated with a       *
*       dollar sign ($).                                                      *
*    Redelivered: 30-NOV-94 by J. Doggett,                                    *
*       GetImage Version 1.1.                                                 *
******************************************************************************/
/*******************************
* Read_Plate_Info declarations *
*******************************/
PLATE Read_Plate_Info( InputFile )
FILE*   InputFile;
{
   PLATE Plate; /* Place for plate data read in */

/* Variables for reading data */
   char   Region  [ 7 ] ;
   char   Plate_Id[ 5 ] ;
   int    RA_Hours      ; /* Plate center                       */
   int    RA_Minutes    ;
   double RA_Seconds    ;
   char   Dec_Sign      ;
   int    Dec_Degrees   ;
   int    Dec_ArcMinutes;
   double Dec_ArcSeconds;
   double RA_Radians    ;
   double Dec_Radians   ;
   double Epoch         ;
   char   Source[ 4 ]   ;
   int    Disc          ;


/*************************
* Read_Plate_Info page 2 *
*************************/

   char   Comment_Character = '#';
   int    Temp_Char     ;
#define PLATE_LIST_EOL_CHAR '$'

/****************************************
* Read_Plate_Info begin executable code *
****************************************/
/* Read plate region, id */
   Region[0] = Comment_Character;
   while ( Region[0] == Comment_Character)
   {
      Region[0] = getc(InputFile);
      if (Region[0] == Comment_Character)
         while (( (Temp_Char=getc(InputFile)) != PLATE_LIST_EOL_CHAR)
                            &&
                (Temp_Char                    != EOF )
                  ) ;
      else
      {
         ungetc(Region[0], InputFile);
         switch(fscanf(InputFile, "%s", Region))
         {
            case 1:
               break;
            case EOF:
               return ( PLATE )EOF;
            default:
	       fprintf(stderr,"Error reading plate id.\n");
	       return (PLATE)NULL;
         }
      }
   }

   switch(fscanf(InputFile, "%s", Plate_Id))
   {
      case 1:
         break;
      case EOF:
         return ( PLATE )EOF;
      default:
         while (( (Temp_Char=getc(InputFile)) != PLATE_LIST_EOL_CHAR)
                            &&
                (Temp_Char                    != EOF )
               ) ;
         return ( PLATE )NULL;
   }

/* Read plate position */
   switch( Read_DMS( InputFile, &RA_Radians, &Dec_Radians ) )
   {
      case 1:
         break;
      case EOF:
         return ( PLATE )EOF;
      default:
         while (( (Temp_Char=getc(InputFile)) != PLATE_LIST_EOL_CHAR)
                            &&
                (Temp_Char                    != EOF )
               ) ;
         return ( PLATE )NULL;

/*************************
* Read_Plate_Info page 3 *
*************************/

   }

/* Read epoch, source, disc */
   switch( fscanf( InputFile, "%lf %s %d",
                   &Epoch, Source, &Disc ) )
   {
   case 3  :
      while (( (Temp_Char=getc(InputFile)) != PLATE_LIST_EOL_CHAR)
                         &&
             (Temp_Char                    != EOF )
            ) ;
      Plate = Create_Plate
            ( Plate_Id, Region, RA_Radians, Dec_Radians, Epoch, Source, Disc );
      return Plate;
   case EOF:
      return ( PLATE )EOF;
   default:
      while (( (Temp_Char=getc(InputFile)) != PLATE_LIST_EOL_CHAR)
                         &&
             (Temp_Char                    != EOF )
            ) ;
      return ( PLATE )NULL;
   }
}
/**************************************
* Read_Plate_Info end executable code *
**************************************/
